module Example where
import Prelude hiding (map, span, div, head)
import HTMLPrelude

import Maybe

doc = 
  build_document (doc_head ## doc_body)

doc_head :: (AddTo a HEAD) => ELT a -> ELT a
doc_head =
  head (title (text "Hello World"))

doc_body :: (AddTo a BODY) => ELT a -> ELT a
doc_body =
  body (h1 (text ttl)
       ## text ("This is the traditional \"" ++ ttl ++ "\" page.")
       ## hr empty
       ## address (hlink (text "Peter Thiemann") "mailto:thiemann@acm.org"))
  where ttl = "Hello World!"

-- 
doc_head_meta :: (AddTo a HEAD) => ELT a -> ELT a
doc_head_meta =
  head (title (text "Hello World") ## meta (attr NAME "Author" ## attr CONTENT "fr"))
-- 
hlink body url =
  a (body ## attr HREF url)

-- 

html_doc ttl contents =
  build_document 
    (head (title (text ttl))
    ## body (h1 (text ttl) ## contents))

signature name url =
  hr empty
  ## address (hlink (text name) url)

my_doc ttl contents =
  html_doc ttl
    (contents
    ## signature "Peter Thiemann" "mailto:thiemann@acm.org")

-- experimental
type ET t = ELT t -> ELT t

data Zero = Zero
data Succ n = Succ n
class Header i h | i -> h where
  h :: AddTo s h => i -> ET h -> ET s

instance Header Zero H1 where
  h Zero = h1
instance Header (Succ Zero) H2 where
  h (Succ Zero) = h2
instance Header (Succ (Succ Zero)) H3 where
  h (Succ (Succ Zero)) = h3
instance Header (Succ (Succ (Succ Zero))) H4 where
  h (Succ (Succ (Succ Zero))) = h4
instance Header (Succ (Succ (Succ (Succ Zero)))) H5 where
  h (Succ (Succ (Succ (Succ Zero)))) = h5
instance Header (Succ (Succ (Succ (Succ (Succ Zero))))) H6 where
  h (Succ (Succ (Succ (Succ (Succ Zero))))) = h6

html_doci i ttl contents =
  build_document 
    (head (title (text ttl))
    ## body (h i (text ttl) ## contents))

{- does not type check
procDoc :: (AddTo s CDATA, Header i h, AddTo h CDATA) => i -> Int -> ET s
procDoc i n =
  if n > 6 then text "limit reached"
           else h i (text "Level" ## text (show n))
	     ## procDoc (Succ i) (n+1)
-}
